@@ -503,6 +503,9 @@ public:
 
 
     Index64 memUsage() const;
+#if OPENVDB_ABI_VERSION_NUMBER >= 10
+    Index64 memUsageIfLoaded() const;
+#endif
 
     void evalActiveBoundingBox(CoordBBox& bbox, bool visitVoxels = true) const;
 
@@ -1169,7 +1172,7 @@ PointDataLeafNode<T, Log2Dim>::readBuffers(std::istream& is, const CoordBBox& /*
             std::string key("paged:" + std::to_string(index));
             auto it = auxData.find(key);
             if (it != auxData.end()) {
-                return *(boost::any_cast<compression::PagedInputStream::Ptr>(it->second));
+                return *(std::any_cast<compression::PagedInputStream::Ptr>(it->second));
             }
             else {
                 compression::PagedInputStream::Ptr pagedStream = std::make_shared<compression::PagedInputStream>();
@@ -1213,7 +1216,7 @@ PointDataLeafNode<T, Log2Dim>::readBuffers(std::istream& is, const CoordBBox& /*
             std::string descriptorKey("descriptorPtr");
             auto itDescriptor = auxData.find(descriptorKey);
             assert(itDescriptor != auxData.end());
-            const Descriptor::Ptr descriptor = boost::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
+            const Descriptor::Ptr descriptor = std::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
             return descriptor;
         }
     };
@@ -1335,7 +1338,7 @@ PointDataLeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
             std::string key("paged:" + std::to_string(index));
             auto it = auxData.find(key);
             if (it != auxData.end()) {
-                compression::PagedOutputStream& stream = *(boost::any_cast<compression::PagedOutputStream::Ptr>(it->second));
+                compression::PagedOutputStream& stream = *(std::any_cast<compression::PagedOutputStream::Ptr>(it->second));
                 stream.flush();
                 (const_cast<io::StreamMetadata::AuxDataMap&>(auxData)).erase(it);
             }
@@ -1347,7 +1350,7 @@ PointDataLeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
             std::string key("paged:" + std::to_string(index));
             auto it = auxData.find(key);
             if (it != auxData.end()) {
-                return *(boost::any_cast<compression::PagedOutputStream::Ptr>(it->second));
+                return *(std::any_cast<compression::PagedOutputStream::Ptr>(it->second));
             }
             else {
                 compression::PagedOutputStream::Ptr pagedStream = std::make_shared<compression::PagedOutputStream>();
@@ -1371,12 +1374,12 @@ PointDataLeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
             }
             else {
                 // if matching bool is found and is false, early exit (a previous descriptor did not match)
-                bool matching = boost::any_cast<bool>(itMatching->second);
+                bool matching = std::any_cast<bool>(itMatching->second);
                 if (!matching)    return;
                 assert(itDescriptor != auxData.end());
                 // if matching bool is true, check whether the existing descriptor matches the current one and set
                 // matching bool to false if not
-                const Descriptor::Ptr existingDescriptor = boost::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
+                const Descriptor::Ptr existingDescriptor = std::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
                 if (*existingDescriptor != *descriptor) {
                     (const_cast<io::StreamMetadata::AuxDataMap&>(auxData))[matchingKey] = false;
                 }
@@ -1390,7 +1393,7 @@ PointDataLeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
             // if matching key is not found, no matching descriptor
             if (itMatching == auxData.end())                return false;
             // if matching key is found and is false, no matching descriptor
-            if (!boost::any_cast<bool>(itMatching->second)) return false;
+            if (!std::any_cast<bool>(itMatching->second)) return false;
             return true;
         }
 
@@ -1401,7 +1404,7 @@ PointDataLeafNode<T, Log2Dim>::writeBuffers(std::ostream& os, bool toHalf) const
             // if matching key is true, however descriptor is not found, it has already been retrieved
             if (itDescriptor == auxData.end())              return nullptr;
             // otherwise remove it and return it
-            const Descriptor::Ptr descriptor = boost::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
+            const Descriptor::Ptr descriptor = std::any_cast<AttributeSet::Descriptor::Ptr>(itDescriptor->second);
             (const_cast<io::StreamMetadata::AuxDataMap&>(auxData)).erase(itDescriptor);
             return descriptor;
         }
@@ -1520,6 +1523,15 @@ PointDataLeafNode<T, Log2Dim>::memUsage() const
     return BaseLeaf::memUsage() + mAttributeSet->memUsage();
 }
 
+#if OPENVDB_ABI_VERSION_NUMBER >= 10
+template<typename T, Index Log2Dim>
+inline Index64
+PointDataLeafNode<T, Log2Dim>::memUsageIfLoaded() const
+{
+    return BaseLeaf::memUsageIfLoaded() + mAttributeSet->memUsageIfLoaded();
+}
+#endif
+
 template<typename T, Index Log2Dim>
 inline void
 PointDataLeafNode<T, Log2Dim>::evalActiveBoundingBox(CoordBBox& bbox, bool visitVoxels) const
