@@ -62,14 +62,14 @@
 #ifndef OPENVDB_TOOLS_PARTICLES_TO_LEVELSET_HAS_BEEN_INCLUDED
 #define OPENVDB_TOOLS_PARTICLES_TO_LEVELSET_HAS_BEEN_INCLUDED
 
-#include "openvdb/Types.h"
-#include "openvdb/Grid.h"
-#include "openvdb/math/Math.h"
-#include "openvdb/math/Transform.h"
-#include "openvdb/tree/LeafManager.h"
-#include "openvdb/util/logging.h"
-#include "openvdb/util/NullInterrupter.h"
-#include "openvdb/thread/Threading.h"
+#include <openvdb/Types.h>
+#include <openvdb/Grid.h>
+#include <openvdb/math/Math.h>
+#include <openvdb/math/Transform.h>
+#include <openvdb/tree/LeafManager.h>
+#include <openvdb/util/logging.h>
+#include <openvdb/util/NullInterrupter.h>
+#include <openvdb/thread/Threading.h>
 
 #include "Composite.h" // for csgUnion()
 #include "PointPartitioner.h"
@@ -686,7 +686,7 @@ private:
                 const Vec3R Nrml = -V * invSpeed; // inverse normalized direction
                 Vec3R P = P0; // local position of instance
                 Real R = R0, d = 0; // local radius and length of trail
-                for (size_t m = 0; run && d <= speed ; ++m) {
+                while (run && d <= speed) {
                     run = this->makeSphere(P, R, att, acc);
                     P += 0.5 * delta * R * Nrml; // adaptive offset along inverse velocity direction
                     d = (P - P0).length(); // current length of trail
